@@ -41,6 +41,8 @@
 #include <functional>
 #endif
 #include "tools/TypesafePtr.h"
+#include "tools/Log.h"
+#include "tools/Tools.h"
 
 
 static bool getenvTypesafeDebug() noexcept {
@@ -79,6 +81,24 @@ extern "C" void*plumed_plumedmain_create() {
   }
 }
 
+extern "C" unsigned plumed_plumedmain_create_reference(void*plumed) {
+  plumed_massert(plumed,"trying to create a reference to a plumed object which is not initialized");
+  auto p=static_cast<PLMD::PlumedMain*>(plumed);
+  return p->increaseReferenceCounter();
+}
+
+extern "C" unsigned plumed_plumedmain_delete_reference(void*plumed) {
+  plumed_massert(plumed,"trying to delete a reference to a plumed object which is not initialized");
+  auto p=static_cast<PLMD::PlumedMain*>(plumed);
+  return p->decreaseReferenceCounter();
+}
+
+extern "C" unsigned plumed_plumedmain_use_count(void*plumed) {
+  plumed_massert(plumed,"trying to delete a reference to a plumed object which is not initialized");
+  auto p=static_cast<PLMD::PlumedMain*>(plumed);
+  return p->useCountReferenceCounter();
+}
+
 extern "C" void plumed_plumedmain_cmd(void*plumed,const char*key,const void*val) {
   plumed_massert(plumed,"trying to use a plumed object which is not initialized");
   auto p=static_cast<PLMD::PlumedMain*>(plumed);
@@ -94,6 +114,160 @@ extern "C" {
   }
 }
 
+/// Internal tool
+/// Throws the currently managed exception and call the nothrow handler.
+/// If nested is not null, it is passed and then gets populated with a pointer that should
+/// be called on the nested exception
+/// If msg is not null, it overrides the message. Can be used to build a concatenated message.
+static void translate_current(plumed_nothrow_handler_x nothrow,void**nested=nullptr,const char*msg=nullptr) {
+  const void* opt[5]= {"n",nested,nullptr,nullptr,nullptr};
+  try {
+    // this function needs to be called while catching an exception
+    // cppcheck-suppress rethrowNoCurrentException
+    throw;
+  } catch(const PLMD::ExceptionTypeError & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,20300,msg,opt);
+  } catch(const PLMD::ExceptionError & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,20200,msg,opt);
+  } catch(const PLMD::ExceptionDebug & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,20100,msg,opt);
+  } catch(const PLMD::Exception & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,20000,msg,opt);
+  } catch(const PLMD::lepton::Exception & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,19900,msg,opt);
+    // 11000 to 12000 are "bad exceptions". message will be copied without new allocations
+  } catch(const std::bad_exception & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,11500,msg,opt);
+#ifdef __PLUMED_LIBCXX11
+  } catch(const std::bad_array_new_length & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,11410,msg,opt);
+#endif
+  } catch(const std::bad_alloc & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,11400,msg,opt);
+#ifdef __PLUMED_LIBCXX11
+  } catch(const std::bad_function_call & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,11300,msg,opt);
+  } catch(const std::bad_weak_ptr & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,11200,msg,opt);
+#endif
+  } catch(const std::bad_cast & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,11100,msg,opt);
+  } catch(const std::bad_typeid & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,11000,msg,opt);
+    // not implemented yet: std::regex_error
+    // we do not allow regex yet due to portability problems with gcc 4.8
+    // as soon as we transition to using <regex> it should be straightforward to add
+  } catch(const std::ios_base::failure & e) {
+    if(!msg) msg=e.what();
+#ifdef __PLUMED_LIBCXX11
+    int value=e.code().value();
+    opt[2]="c"; // "c" passes the error code.
+    opt[3]=&value;
+    if(e.code().category()==std::generic_category()) nothrow.handler(nothrow.ptr,10230,msg,opt);
+    else if(e.code().category()==std::system_category()) nothrow.handler(nothrow.ptr,10231,msg,opt);
+    else if(e.code().category()==std::iostream_category()) nothrow.handler(nothrow.ptr,10232,msg,opt);
+    else if(e.code().category()==std::future_category()) nothrow.handler(nothrow.ptr,10233,msg,opt);
+    else
+#endif
+      // 10239 represents std::ios_base::failure with default constructur
+      nothrow.handler(nothrow.ptr,10239,msg,opt);
+#ifdef __PLUMED_LIBCXX11
+  } catch(const std::system_error & e) {
+    if(!msg) msg=e.what();
+    int value=e.code().value();
+    opt[2]="c"; // "c" passes the error code.
+    opt[3]=&value;
+    if(e.code().category()==std::generic_category()) nothrow.handler(nothrow.ptr,10220,msg,opt);
+    else if(e.code().category()==std::system_category()) nothrow.handler(nothrow.ptr,10221,msg,opt);
+    else if(e.code().category()==std::iostream_category()) nothrow.handler(nothrow.ptr,10222,msg,opt);
+    else if(e.code().category()==std::future_category()) nothrow.handler(nothrow.ptr,10223,msg,opt);
+    // fallback to generic runtime_error
+    else nothrow.handler(nothrow.ptr,10200,msg,opt);
+#endif
+  } catch(const std::underflow_error &e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10215,msg,opt);
+  } catch(const std::overflow_error &e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10210,msg,opt);
+  } catch(const std::range_error &e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10205,msg,opt);
+  } catch(const std::runtime_error & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10200,msg,opt);
+    // not implemented yet: std::future_error
+    // not clear how useful it would be.
+  } catch(const std::out_of_range & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10120,msg,opt);
+  } catch(const std::length_error & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10115,msg,opt);
+  } catch(const std::domain_error & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10110,msg,opt);
+  } catch(const std::invalid_argument & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10105,msg,opt);
+  } catch(const std::logic_error & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10100,msg,opt);
+    // generic exception. message will be copied without new allocations
+    // reports all non caught exceptions that are derived from std::exception
+    // for instance, boost exceptions would end up here
+  } catch(const std::exception & e) {
+    if(!msg) msg=e.what();
+    nothrow.handler(nothrow.ptr,10000,msg,opt);
+  } catch(const char* m) {
+    if(!msg) msg=m;
+    nothrow.handler(nothrow.ptr,10000,msg,opt);
+  } catch(const std::string & s) {
+    if(!msg) msg=s.c_str();
+    nothrow.handler(nothrow.ptr,10000,msg,opt);
+  } catch (...) {
+    // if exception cannot be translated, we add a bad_exception to the stack
+    nothrow.handler(nothrow.ptr,11500,"plumed could not translate exception",opt);
+  }
+}
+
+static void translate_nested(plumed_nothrow_handler_x nothrow) {
+  try {
+    throw;
+  } catch (const std::nested_exception & e) {
+// If this exception has a nested one:
+    auto nothrow_nested=nothrow;
+    nothrow_nested.ptr=nullptr;
+// translate the current exception asking the wrapper to allocate a new exception
+    translate_current(nothrow,&nothrow_nested.ptr);
+// if the wrapper cannot allocate the exception, this will be a nullptr
+    if(nothrow_nested.ptr) {
+      try {
+// transfer control to the nested exception
+        e.rethrow_nested();
+      } catch (...) {
+// recursively translate it
+        translate_nested(nothrow_nested);
+      }
+    }
+  } catch (...) {
+// otherwise, just translate the current exception
+    translate_current(nothrow);
+  }
+}
+
 extern "C" {
   static void plumed_plumedmain_cmd_safe_nothrow(void*plumed,const char*key,plumed_safeptr_x safe,plumed_nothrow_handler_x nothrow) {
 // This is a workaround for a suboptimal choice in PLUMED <2.8
@@ -105,98 +279,24 @@ extern "C" {
       plumed_plumedmain_cmd_safe(plumed,key,safe);
       return;
     }
+    auto p=static_cast<PLMD::PlumedMain*>(plumed);
 // At library boundaries we translate exceptions to error codes.
 // This allows an exception to be catched also if the MD code
 // was linked against a different C++ library
     try {
       plumed_massert(plumed,"trying to use a plumed object which is not initialized");
-      auto p=static_cast<PLMD::PlumedMain*>(plumed);
       if(getenvTypesafeDebug()) typesafeDebug(key,safe);
       p->cmd(key,PLMD::TypesafePtr::fromSafePtr(&safe));
-    } catch(const PLMD::ExceptionTypeError & e) {
-      nothrow.handler(nothrow.ptr,20300,e.what(),nullptr);
-    } catch(const PLMD::ExceptionError & e) {
-      nothrow.handler(nothrow.ptr,20200,e.what(),nullptr);
-    } catch(const PLMD::ExceptionDebug & e) {
-      nothrow.handler(nothrow.ptr,20100,e.what(),nullptr);
-    } catch(const PLMD::Exception & e) {
-      nothrow.handler(nothrow.ptr,20000,e.what(),nullptr);
-    } catch(const PLMD::lepton::Exception & e) {
-      nothrow.handler(nothrow.ptr,19900,e.what(),nullptr);
-      // 11000 to 12000 are "bad exceptions". message will be copied without new allocations
-    } catch(const std::bad_exception & e) {
-      nothrow.handler(nothrow.ptr,11500,e.what(),nullptr);
-#ifdef __PLUMED_LIBCXX11
-    } catch(const std::bad_array_new_length & e) {
-      nothrow.handler(nothrow.ptr,11410,e.what(),nullptr);
-#endif
-    } catch(const std::bad_alloc & e) {
-      nothrow.handler(nothrow.ptr,11400,e.what(),nullptr);
-#ifdef __PLUMED_LIBCXX11
-    } catch(const std::bad_function_call & e) {
-      nothrow.handler(nothrow.ptr,11300,e.what(),nullptr);
-    } catch(const std::bad_weak_ptr & e) {
-      nothrow.handler(nothrow.ptr,11200,e.what(),nullptr);
-#endif
-    } catch(const std::bad_cast & e) {
-      nothrow.handler(nothrow.ptr,11100,e.what(),nullptr);
-    } catch(const std::bad_typeid & e) {
-      nothrow.handler(nothrow.ptr,11000,e.what(),nullptr);
-      // not implemented yet: std::regex_error
-      // we do not allow regex yet due to portability problems with gcc 4.8
-      // as soon as we transition to using <regex> it should be straightforward to add
-    } catch(const std::ios_base::failure & e) {
-#ifdef __PLUMED_LIBCXX11
-      int value=e.code().value();
-      const void* opt[3]= {"c",&value,nullptr}; // "c" passes the error code. nullptr terminates the optional part.
-      if(e.code().category()==std::generic_category()) nothrow.handler(nothrow.ptr,10230,e.what(),opt);
-      else if(e.code().category()==std::system_category()) nothrow.handler(nothrow.ptr,10231,e.what(),opt);
-      else if(e.code().category()==std::iostream_category()) nothrow.handler(nothrow.ptr,10232,e.what(),opt);
-      else if(e.code().category()==std::future_category()) nothrow.handler(nothrow.ptr,10233,e.what(),opt);
-      else
-#endif
-        // 10239 represents std::ios_base::failure with default constructur
-        nothrow.handler(nothrow.ptr,10239,e.what(),nullptr);
-#ifdef __PLUMED_LIBCXX11
-    } catch(const std::system_error & e) {
-      int value=e.code().value();
-      const void* opt[3]= {"c",&value,nullptr}; // "c" passes the error code. nullptr terminates the optional part.
-      if(e.code().category()==std::generic_category()) nothrow.handler(nothrow.ptr,10220,e.what(),opt);
-      else if(e.code().category()==std::system_category()) nothrow.handler(nothrow.ptr,10221,e.what(),opt);
-      else if(e.code().category()==std::iostream_category()) nothrow.handler(nothrow.ptr,10222,e.what(),opt);
-      else if(e.code().category()==std::future_category()) nothrow.handler(nothrow.ptr,10223,e.what(),opt);
-      // fallback to generic runtime_error
-      else nothrow.handler(nothrow.ptr,10200,e.what(),nullptr);
-#endif
-    } catch(const std::underflow_error &e) {
-      nothrow.handler(nothrow.ptr,10215,e.what(),nullptr);
-    } catch(const std::overflow_error &e) {
-      nothrow.handler(nothrow.ptr,10210,e.what(),nullptr);
-    } catch(const std::range_error &e) {
-      nothrow.handler(nothrow.ptr,10205,e.what(),nullptr);
-    } catch(const std::runtime_error & e) {
-      nothrow.handler(nothrow.ptr,10200,e.what(),nullptr);
-      // not implemented yet: std::future_error
-      // not clear how useful it would be.
-    } catch(const std::out_of_range & e) {
-      nothrow.handler(nothrow.ptr,10120,e.what(),nullptr);
-    } catch(const std::length_error & e) {
-      nothrow.handler(nothrow.ptr,10115,e.what(),nullptr);
-    } catch(const std::domain_error & e) {
-      nothrow.handler(nothrow.ptr,10110,e.what(),nullptr);
-    } catch(const std::invalid_argument & e) {
-      nothrow.handler(nothrow.ptr,10105,e.what(),nullptr);
-    } catch(const std::logic_error & e) {
-      nothrow.handler(nothrow.ptr,10100,e.what(),nullptr);
-      // generic exception. message will be copied without new allocations
-      // reports all non caught exceptions that are derived from std::exception
-      // for instance, boost exceptions would end up here
-    } catch(const std::exception & e) {
-      nothrow.handler(nothrow.ptr,10000,e.what(),nullptr);
     } catch(...) {
-      // if exception cannot be translated, we throw a bad_exception
-      nothrow.handler(nothrow.ptr,11500,"plumed could not translate exception",nullptr);
-      throw;
+      if(p->getNestedExceptions()) {
+        translate_nested(nothrow);
+      } else {
+// In this case, we just consider the latest thrown exception and
+// supplement it with a concatenated message so as not to
+// loose information.
+        auto msg=PLMD::Tools::concatenateExceptionMessages();
+        translate_current(nothrow,nullptr,msg.c_str());
+      }
     }
   }
 }
@@ -223,22 +323,28 @@ extern "C" void plumed_plumedmain_finalize(void*plumed) {
 
 // values here should be consistent with those in plumed_symbol_table_init !!!!
 plumed_symbol_table_type_x plumed_symbol_table= {
-  3,
+  4,
   {plumed_plumedmain_create,plumed_plumedmain_cmd,plumed_plumedmain_finalize},
   plumed_plumedmain_cmd_nothrow,
   plumed_plumedmain_cmd_safe,
-  plumed_plumedmain_cmd_safe_nothrow
+  plumed_plumedmain_cmd_safe_nothrow,
+  plumed_plumedmain_create_reference,
+  plumed_plumedmain_delete_reference,
+  plumed_plumedmain_use_count
 };
 
 // values here should be consistent with those above !!!!
 extern "C" void plumed_symbol_table_init() {
-  plumed_symbol_table.version=3;
+  plumed_symbol_table.version=4;
   plumed_symbol_table.functions.create=plumed_plumedmain_create;
   plumed_symbol_table.functions.cmd=plumed_plumedmain_cmd;
   plumed_symbol_table.functions.finalize=plumed_plumedmain_finalize;
   plumed_symbol_table.cmd_nothrow=plumed_plumedmain_cmd_nothrow;
   plumed_symbol_table.cmd_safe=plumed_plumedmain_cmd_safe;
   plumed_symbol_table.cmd_safe_nothrow=plumed_plumedmain_cmd_safe_nothrow;
+  plumed_symbol_table.create_reference=plumed_plumedmain_create_reference;
+  plumed_symbol_table.delete_reference=plumed_plumedmain_delete_reference;
+  plumed_symbol_table.use_count=plumed_plumedmain_use_count;
 }
 
 namespace PLMD {
